package ru.ldeloff.servermonitorbot.service.command.mdadm;

import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;
import org.telegram.telegrambots.meta.api.methods.send.SendMessage;
import ru.ldeloff.servermonitorbot.model.Command;
import ru.ldeloff.servermonitorbot.model.SshServer;
import ru.ldeloff.servermonitorbot.model.User;
import ru.ldeloff.servermonitorbot.service.command.CommandTemplate;
import ru.ldeloff.servermonitorbot.service.role.RoleService;
import ru.ldeloff.servermonitorbot.service.ssh.SshService;
import ru.ldeloff.servermonitorbot.service.user.UserService;
import ru.ldeloff.servermonitorbot.utils.SshServerUtils;

import java.util.*;
import java.util.regex.Matcher;
import java.util.regex.Pattern;
import java.util.stream.Collectors;

@Service
@Slf4j
public class MdadmStatusCommand extends CommandTemplate {

    final SshService sshService;

    public MdadmStatusCommand(UserService userService, RoleService roleService, SshService sshService) {
        super(userService, roleService);
        this.sshService = sshService;
    }

    @Override
    public SendMessage actionForAuth(User user, SendMessage message) {
        String [] tags = Arrays.stream(message.getText().split(":"))
                .map(String::trim)
                .toArray(String[]::new);

        if (tags[1].equals("all")) {
            message.setText(mdadmStatusAllHost());
        } else {
            message.setText(Objects.requireNonNull(mdadmStatusSpecificHost(tags[1])));
        }
        return message;
    }

    private String mdadmStatusAllHost() {
        List<SshServer> servers = SshServerUtils.filterGoodServers(sshService.getSshServers());
        StringBuilder response = new StringBuilder();
        servers.forEach(server -> response.append(mdadmStatusSpecificHost(server.getName())).append("\n"));
        return response.toString();
    }

    private String mdadmStatusSpecificHost(String serverName) {
        Optional<SshServer> server = sshService.getSshServers()
                .stream()
                .filter(x -> x.getName().equals(serverName))
                .findFirst();
        if (server.isPresent()) {
            // узнаю скок рейдов
            Command commandRaid = sshService.execute(new Command("mdadm --detail --scan", 500, server.get()));
            List<String> raidNames = parseRaidNames(commandRaid);
            // Для каждого рейда надо выяснить статус
            StringBuilder result = new StringBuilder();
            raidNames.forEach(raid -> {
                result.append(raid).append(":\n");
                Command commandHdd = sshService.execute(new Command("mdadm --detail " + raid, 500, server.get()));
                List<String> hdds = parseHddState(commandHdd);
                hdds.forEach(hdd -> result.append(hdd).append("\n"));
            });
            return server.get().getName() + ": \n" + result;
        } else {
            log.error("Ошибка при выполнении команды 'mdadm'. Искомый сервер (" + serverName + ") не найден");
            return null;
        }
    }
    private List<String> parseRaidNames(Command result) {
        List<String> names = new ArrayList<>();
        Arrays.stream(result.getResponse().split("\n"))
                .map(line -> line.split(" "))
                .filter(lineArray -> lineArray.length >= 2)
                .map(lineArray -> lineArray[1])
                .forEach(names::add);
        return names;
    }

    private List<String> parseHddState(Command result) {
        List<String> names = new ArrayList<>();
        names.add("Number Name State");

        String[] lines = result.getResponse().split("\n");

        boolean foundHeader = false;

        for (String line : lines) {
            if (!foundHeader) {
                if ( line.contains("Number") &&
                        line.contains("Major") &&
                        line.contains("Minor") &&
                        line.contains("RaidDevice") )  {
                    foundHeader = true;
                }
            } else {
                List<String> elements = Arrays.stream(line.split(" "))
                        .filter(x -> !Objects.equals(x, "")).toList();

                names.add(elements.get(0) + ", "
                        + elements.get(6) + ", "
                        + elements.get(4) + " "
                        + elements.get(5));
            }
        }
        return names;
    }
}
